@@ -7,7 +7,7 @@
 # If the MCMC simulation has been run for only one chain report the Geweke diagnostic (Z-score)
 #
 
-Id <- "$Id: c212.interim.1a.hier2.lev1.convergence.R,v 1.6 2016/12/19 11:36:43 clb13102 Exp clb13102 $"
+Id <- "$Id: c212.interim.1a.hier2.lev1.convergence.R,v 1.7 2019/05/05 13:18:12 clb13102 Exp clb13102 $"
 
 c212.interim.1a.hier2.lev1.convergence.diag <- function(raw, debug_diagnostic = FALSE)
 {
@@ -463,12 +463,3 @@ c212.interim.1a.hier2.lev1.print.convergence.summary <- function(conv) {
 		}
 	}
 }
-
-chk_val <- function(val, q = 0.975) {
-	if (abs(val) > qnorm(q)) {
-		return("*")
-	}
-	else {
-		return("-")
-	}
-}
