@@ -6,7 +6,7 @@
 # If the MCMC simulation has been run for only one chain report the Geweke diagnostic (Z-score)
 #
 
-Id <- "$Id: c212.interim.1a.hier3.lev1.convergence.R,v 1.8 2016/12/19 11:36:42 clb13102 Exp clb13102 $"
+Id <- "$Id: c212.interim.1a.hier3.lev1.convergence.R,v 1.9 2019/05/05 13:18:12 clb13102 Exp clb13102 $"
 
 c212.interim.1a.dep.lev1.convergence.diag <- function(raw, debug_diagnostic = FALSE)
 {
@@ -542,7 +542,7 @@ c212.interim.1a.dep.lev1.print.convergence.summary <- function(conv) {
 			cat(sprintf("---------\n"))
 			max_t = head(conv$mu.theta.conv.diag[conv$mu.theta.conv.diag$stat
 						== max(conv$mu.theta.conv.diag$stat),, drop = FALSE], 1)
-			cat(sprintf("Max Geweke statistic (%s %s): %0.6f (%s)\n", max_t$Interval, max_t$B, max_t$stat,
+			cat(sprintf("Max Geweke statistic (%s): %0.6f (%s)\n", max_t$B, max_t$stat,
 												chk_val(max_t$stat)))
 			min_t = head(conv$mu.theta.conv.diag[conv$mu.theta.conv.diag$stat
 						== min(conv$mu.theta.conv.diag$stat),, drop = FALSE], 1)
@@ -647,12 +647,3 @@ c212.interim.1a.dep.lev1.print.convergence.summary <- function(conv) {
 		}
 	}
 }
-
-chk_val <- function(val, q = 0.975) {
-	if (abs(val) > qnorm(q)) {
-		return("*")
-	}
-	else {
-		return("-")
-	}
-}
